# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from typing import Dict, List, Optional, TypeVar
import urllib.parse

from eventline.account import Account
from eventline.api_object import ReadableAPIObject
from eventline.client import Client, Response
from eventline.command_execution import CommandExecutionInput, CommandExecution
from eventline.event import Event, NewEvent
from eventline.organization import Organization
from eventline.pagination import Cursor, Page
from eventline.pipeline import Pipeline
from eventline.project import Project, NewProject, ProjectUpdate
from eventline.resource import Resource
from eventline.task import Task

ResponseObjectType = TypeVar("ResponseObjectType", bound=ReadableAPIObject)


class APIClient(Client):
    """A high level API client for the Eventline API."""

    def get_current_organization(self) -> Organization:
        """Fetch the organization associated with the credentials currently
        used by the client."""
        response = self.send_request("GET", "/org")
        return read_response(response, Organization())

    def get_current_account(self) -> Account:
        """Fetch the account associated with the credentials currently used
        by the client."""
        response = self.send_request("GET", "/account")
        return read_response(response, Account())

    def get_accounts(self, /, cursor: Optional[Cursor] = None) -> Page:
        """Fetch all accounts in the organization."""
        response = self.send_request("GET", "/accounts", cursor=cursor)
        return read_response(response, Page(Account))

    def get_account(self, id_: str) -> Account:
        """Fetch an account by identifier."""
        response = self.send_request("GET", f"/accounts/id/{path_escape(id_)}")
        return read_response(response, Account())

    def get_projects(self, /, cursor: Optional[Cursor] = None) -> Page:
        """Fetch projects in the organization."""
        response = self.send_request("GET", "/projects", cursor=cursor)
        return read_response(response, Page(Project))

    def create_project(self, new_project: NewProject) -> Project:
        """Create a new project."""
        body = new_project._serialize()
        response = self.send_request("POST", "/projects", body=body)
        return read_response(response, Project())

    def get_project(self, id_: str) -> Project:
        """Fetch a project by identifier."""
        response = self.send_request("GET", f"/projects/id/{path_escape(id_)}")
        return read_response(response, Project())

    def get_project_by_name(self, name: str) -> Project:
        """Fetch a project by name."""
        response = self.send_request(
            "GET", f"/projects/name/{path_escape(name)}"
        )
        return read_response(response, Project())

    def update_project(
        self, id_: str, project_update: ProjectUpdate
    ) -> Project:
        """Update an existing project."""
        body = project_update._serialize()
        response = self.send_request(
            "PUT", f"/projects/id/{path_escape(id_)}", body=body
        )
        return read_response(response, Project())

    def delete_project(self, id_: str) -> None:
        """Delete a project."""
        self.send_request("DELETE", f"/projects/id/{path_escape(id_)}")

    def get_resources(self, /, cursor: Optional[Cursor] = None) -> Page:
        """Fetch resources in the project."""
        response = self.send_request("GET", "/resources", cursor=cursor)
        return read_response(response, Page(Resource))

    def get_resource(self, id_: str) -> Resource:
        """Fetch a resource by identifier."""
        response = self.send_request(
            "GET", f"/resources/id/{path_escape(id_)}"
        )
        return read_response(response, Resource())

    def get_resource_by_name(self, type_: str, name: str) -> Resource:
        """Fetch a resource by type and name."""
        response = self.send_request(
            "GET",
            f"/resources/type/{path_escape(type_)}/name/{path_escape(name)}",
        )
        return read_response(response, Resource())

    def execute_command(
        self, id_: str, input_: CommandExecutionInput
    ) -> CommandExecution:
        """Execute a command and return the associated command execution
        object."""
        body = input_._serialize()
        response = self.send_request(
            "POST", f"/commands/id/{path_escape(id_)}/execute", body=body
        )
        return read_response(response, CommandExecution())

    def get_command_executions(
        self,
        /,
        command_id: Optional[str] = None,
        cursor: Optional[Cursor] = None,
    ) -> Page:
        """Fetch a list of command executions."""
        query_parameters = {}
        if command_id is not None:
            query_parameters["command"] = command_id
        response = self.send_request(
            "GET",
            "/command_executions",
            query_parameters=query_parameters,
            cursor=cursor,
        )
        return read_response(response, Page(CommandExecution))

    def get_command_execution(self, id_: str) -> CommandExecution:
        """Fetch a command execution by identifier."""
        response = self.send_request(
            "GET", f"/command_executions/id/{path_escape(id_)}"
        )
        return read_response(response, CommandExecution())

    def create_event(self, new_event: NewEvent) -> List[Event]:
        """Create a new custom event."""
        body = new_event._serialize()
        response = self.send_request("POST", "/events", body=body)
        events = []
        for value_object in response.body:
            value = Event()
            value._read(value_object)
            events.append(value)
        return events

    def replay_event(self, id_: str) -> Event:
        """Replay an existing event."""
        response = self.send_request(
            "POST", f"/events/id/{path_escape(id_)}/replay"
        )
        return read_response(response, Event())

    def get_pipelines(self, /, cursor: Optional[Cursor] = None) -> Page:
        """Fetch pipelines in the project."""
        response = self.send_request("GET", "/pipelines", cursor=cursor)
        return read_response(response, Page(Pipeline))

    def get_pipeline(self, id_: str) -> Pipeline:
        """Fetch a pipeline by identifier."""
        response = self.send_request(
            "GET", f"/pipelines/id/{path_escape(id_)}"
        )
        return read_response(response, Pipeline())

    def abort_pipeline(self, id_: str) -> None:
        """Abort a pipeline."""
        self.send_request("POST", f"/pipelines/id/{path_escape(id_)}/abort")

    def restart_pipeline(self, id_: str) -> None:
        """Restart a finished pipeline."""
        self.send_request("POST", f"/pipelines/id/{path_escape(id_)}/restart")

    def restart_pipeline_from_failure(self, id_: str) -> None:
        """Restart the failed tasks in a finished pipeline."""
        self.send_request(
            "POST", f"/pipelines/id/{path_escape(id_)}/restart_from_failure"
        )

    def get_scratchpad(self, id_: str) -> Dict[str, str]:
        """Fetch all entries in a scratchpad."""
        response = self.send_request(
            "GET", f"/pipelines/id/{path_escape(id_)}/scratchpad"
        )
        return response.body

    def delete_scratchpad(self, id_: str) -> None:
        """Delete all entries in a scratchpad."""
        self.send_request(
            "DELETE", f"/pipelines/id/{path_escape(id_)}/scratchpad"
        )

    def get_scratchpad_entry(self, id_: str, key: str) -> str:
        """Fetch an entry in a scratchpad."""
        response = self.send_request(
            "GET",
            f"/pipelines/id/{path_escape(id_)}/scratchpad"
            f"/key/{path_escape(key)}",
        )
        return response.body

    def set_scratchpad_entry(self, id_: str, key: str, value: str) -> None:
        """set the value of an entry in a scratchpad."""
        self.send_request(
            "PUT",
            f"/pipelines/id/{path_escape(id_)}/scratchpad"
            f"/key/{path_escape(key)}",
            body=value,
            raw_body=True,
        )

    def delete_scratchpad_entry(self, id_: str, key: str) -> None:
        """Delete an entry in a scratchpad."""
        self.send_request(
            "DELETE",
            f"/pipelines/id/{path_escape(id_)}/scratchpad"
            f"/key/{path_escape(key)}",
        )

    def get_tasks(self, /, cursor: Optional[Cursor] = None) -> Page:
        """Fetch tasks in the project."""
        response = self.send_request("GET", "/tasks", cursor=cursor)
        return read_response(response, Page(Task))

    def get_task(self, id_: str) -> Task:
        """Fetch a task by identifier."""
        response = self.send_request("GET", f"/tasks/id/{path_escape(id_)}")
        return read_response(response, Task())

    def get_events(self, /, cursor: Optional[Cursor] = None) -> Page:
        """Fetch a list of events."""
        response = self.send_request("GET", "/events", cursor=cursor)
        return read_response(response, Page(Event))

    def get_event(self, id_: str) -> Event:
        """Fetch an event by identifier."""
        response = self.send_request("GET", f"/events/id/{path_escape(id_)}")
        return read_response(response, Event())


def path_escape(string: str) -> str:
    return urllib.parse.quote(string)


def read_response(
    response: Response, value: ResponseObjectType
) -> ResponseObjectType:
    """Read the content of a response and use it to populate an API object."""
    value._read(response.body)
    return value



# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from typing import Any, Dict

from eventline.api_object import ReadableAPIObject


class Organization(ReadableAPIObject):
    """An organization."""

    def __init__(self) -> None:
        super().__init__("organization")

    def _read(self, data: Dict[str, Any]) -> None:
        self.id_ = self._read_string(data, "id")
        self.name = self._read_string(data, "name")
        self.address = self._read_string(data, "address")
        self.postal_code = self._read_string(data, "postal_code")
        self.city = self._read_string(data, "city")
        self.country = self._read_string(data, "country")
        self.creation_time = self._read_datetime(data, "creation_time")
        self.disabled = self._read_optional_boolean(data, "disabled")
        self.contact_email_address = self._read_string(
            data, "contact_email_address"
        )
        self.non_essential_mail_opt_in = self._read_optional_boolean(
            data, "non_essential_mail_opt_in"
        )
        self.vat_id_number = self._read_optional_string(data, "vat_id_number")



#!/usr/bin/env python

# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import argparse
import logging
import pathlib
import sys

import yaml

sys.path.insert(0, str(pathlib.Path(__file__).parents[1]))

import eventline

logging.basicConfig(level=logging.INFO)
eventline.client.log.setLevel(logging.DEBUG)


def string_presenter(dumper, string):
    tag = "tag:yaml.org,2002:str"
    if "\n" in string:
        return dumper.represent_scalar(tag, string, style="|")
    return dumper.represent_scalar(tag, string)


yaml.add_representer(str, string_presenter)

parser = argparse.ArgumentParser(
    description="Fetch and print the specification of a resource."
)
parser.add_argument(
    "--project",
    dest="project",
    type=str,
    default="default",
    help="the name of the project",
)
parser.add_argument(
    "type",
    type=str,
    choices=["trigger", "pipeline", "task", "command"],
    help="the type of the resource",
)
parser.add_argument("name", type=str, help="the name of the resource")
args = parser.parse_args()

client = eventline.APIClient(project_name=args.project)

resource = client.get_resource_by_name(args.type, args.name)
print(yaml.dump(resource.spec))



# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from typing import Any, Dict, Optional

from eventline.api_object import ReadableAPIObject


class Cursor(ReadableAPIObject):
    """A cursor marking a position in a list of paginated objects."""

    def __init__(
        self, /, size: int = 20, sort: str = "id", order: str = "asc"
    ) -> None:
        super().__init__("cursor")
        self.before = None  # type: Optional[str]
        self.after = None  # type: Optional[str]
        self.size = size  # type: Optional[int]
        self.sort = sort  # type: Optional[str]
        self.order = order  # type: Optional[str]

    def _read(self, data: Dict[str, Any]) -> None:
        self.before = self._read_optional_string(data, "before")
        self.after = self._read_optional_string(data, "after")
        self.size = self._read_optional_integer(data, "size")
        self.sort = self._read_optional_string(data, "sort")
        self.order = self._read_optional_string(data, "order")


class Page(ReadableAPIObject):
    """A list of objects."""

    def __init__(self, element_class_type: Any) -> None:
        super().__init__("page")
        self.element_class_type = element_class_type

    def _read(self, data: Dict[str, Any]) -> None:
        self.elements = self._read_object_array(
            data, "elements", self.element_class_type
        )
        self.previous = self._read_optional_object(data, "previous", Cursor)
        self.next = self._read_optional_object(data, "next", Cursor)



# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import datetime
from typing import Any, Optional, Dict

from eventline.api_object import ReadableAPIObject, SerializableAPIObject


class Event(ReadableAPIObject):
    """An event."""

    def __init__(self) -> None:
        super().__init__("event")

    def _read(self, data: Dict[str, Any]) -> None:
        self.id_ = self._read_string(data, "id")
        self.org_id = self._read_string(data, "org_id")
        self.project_id = self._read_string(data, "project_id")
        self.trigger_id = self._read_optional_string(data, "trigger_id")
        self.command_id = self._read_optional_string(data, "command_id")
        self.creation_time = self._read_datetime(data, "creation_time")
        self.event_time = self._read_datetime(data, "event_time")
        self.connector = self._read_string(data, "connector")
        self.name = self._read_string(data, "name")
        self.data = data["data"]
        self.original_event_id = data["original_event_id"]


class NewEvent(SerializableAPIObject):
    """A new event."""

    def __init__(
        self,
        connector: str,
        name: str,
        data: Dict[str, Any],
        /,
        event_time: Optional[datetime.datetime] = None,
    ) -> None:
        super().__init__("new_event")
        self.event_time = event_time
        self.connector = connector
        self.name = name
        self.data = data

    def _serialize(self) -> Dict[str, Any]:
        data = {
            "connector": self.connector,
            "name": self.name,
            "data": self.data,
        }  # type: Dict[str, Any]
        if self.event_time is not None:
            event_time = self.event_time.replace(tzinfo=datetime.timezone.utc)
            data["event_time"] = event_time.strftime("%Y-%m-%dT%H:%M:%SZ")
        return data



#!/usr/bin/env python

# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import argparse
import logging
import pathlib
import sys

sys.path.insert(0, str(pathlib.Path(__file__).parents[1]))

from tabulate import tabulate

import eventline
import eventline.client

logging.basicConfig(level=logging.INFO)
eventline.client.log.setLevel(logging.DEBUG)

parser = argparse.ArgumentParser(
    description="List all accounts in the Eventline organization."
)
parser.parse_args()

client = eventline.APIClient()

table = []

cursor = eventline.Cursor(size=20, sort="name")
while cursor is not None:
    page = client.get_accounts(cursor=cursor)
    for account in page.elements:
        table.append([account.email_address, account.name])
    cursor = page.next

print(tabulate(table, ["Email address", "Name"]))



# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import json
from typing import Any, Dict, Optional

from eventline.api_object import ReadableAPIObject
from eventline.event import Event
import eventline.environment


class Context(ReadableAPIObject):
    """A runtime context."""

    def __init__(self) -> None:
        super().__init__("context")

    def _read(self, data: Dict[str, Any]) -> None:
        self.event = self._read_object(data, "event", Event)
        self.task_parameters = data["task_parameters"]
        self.instance_id = self._read_integer(data, "instance_id")
        self.identities = data["identities"]


def load_context(path: Optional[str] = None) -> Context:
    """Load and return a context object. If the path parameter is None, use the
    EVENTLINE_CONTEXT_PATH environment variable.
    """
    if path is None:
        path = eventline.environment.context_path()
        if path is None:
            raise Exception("no context path available")
    content = None
    with open(path, "r", encoding="UTF-8") as file:
        content = file.read()
    data = json.loads(content)
    context = Context()
    context._read(data)
    return context



# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import os
from typing import Optional


def api_key() -> Optional[str]:
    """Return the API key available in the current environment if there is
    one or None if there is not one.
    """
    return os.environ.get("EVENTLINE_API_KEY")


def project_id() -> Optional[str]:
    """Return the identifier of the current project if it is available in the
    current environment.
    """
    return os.environ.get("EVENTLINE_PROJECT_ID")


def context_path() -> Optional[str]:
    """Return the path of the context file if it is available in the current
    environment."""
    return os.environ.get("EVENTLINE_CONTEXT_PATH")
